package testocl.handlers;

import java.net.URL;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.henshin.interpreter.EGraph;
import org.eclipse.emf.henshin.interpreter.impl.EGraphImpl;
import org.eclipse.emf.henshin.model.resource.HenshinResourceSet;
import org.eclipse.uml2.uml.StateMachine;
import org.eclipse.uml2.uml.UMLPackage;
import org.eclipse.uml2.uml.resource.UMLResource;
import org.osgi.framework.Bundle;


/**
 * Our sample handler extends AbstractHandler, an IHandler base class.
 * @see org.eclipse.core.commands.IHandler
 * @see org.eclipse.core.commands.AbstractHandler
 */
public class SampleHandler extends AbstractHandler {
	private HenshinResourceSet resourceSet;
	private EGraph graph;
	/**
	 * The constructor.
	 */
	public SampleHandler() {
	}

		/**
		 * the command has been executed, so extract extract the needed information
		 * from the application context.
		 */
		public Object execute(ExecutionEvent event) throws ExecutionException {
			loadModel();
			System.out.println("The simple query goes fine:");
			utilOCLQueryPivot("Transition.allInstances()");
			System.out.println("Creating the query using ensureRole fails:");
			utilOCLQueryPivot("Transition.allInstances().extension_ensureRole->excluding(null).role");//This fails
			return null;
		}


		public void loadModel() {
			EPackage.Registry.INSTANCE.put(UMLPackage.eNS_URI, UMLPackage.eINSTANCE);
			Resource.Factory.Registry.INSTANCE.getExtensionToFactoryMap().put(UMLResource.FILE_EXTENSION,
					UMLResource.Factory.INSTANCE);

			// Create a resource set with a base directory:
			resourceSet = new HenshinResourceSet(null);

			resourceSet.getPackageRegistry().put(UMLPackage.eNS_URI, UMLPackage.eINSTANCE);
			resourceSet.registerXMIResourceFactories("uml");
			resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap().put(UMLResource.FILE_EXTENSION,
					UMLResource.Factory.INSTANCE);

			// Load the model into an EGraph:
			Bundle bundle = Platform.getBundle("testocl");


			Path path = new Path("resources/testmodel.uml");

			URL fileURL = FileLocator.find(bundle, path, null);
			Resource resource = resourceSet.getResource(org.eclipse.emf.common.util.URI.createURI(fileURL.toString()),
					true);

			graph = new EGraphImpl(resource);

		}
		
		public void utilOCLQueryPivot(String oclexpression){
			org.eclipse.ocl.pivot.utilities.OCL ocl=org.eclipse.ocl.pivot.utilities.OCL.newInstance(resourceSet);
			
			org.eclipse.ocl.pivot.ExpressionInOCL query;
			try {
				query = ocl.createQuery(getStateMachine().eClass(),oclexpression);
				
				// create a Query to evaluate our query expression
				org.eclipse.ocl.pivot.utilities.Query queryEval = ocl.createQuery(query);	
				Object result=queryEval.evaluateUnboxed(graph.getRoots().get(0));
				System.out.println(result);
			} catch (org.eclipse.ocl.pivot.utilities.ParserException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	

	public StateMachine getStateMachine() {
		StateMachine ret = null;
		for (EObject o : graph.getRoots().get(0).eContents()) {
			if (o instanceof StateMachine) {
				ret = (StateMachine) o;
				break;
			}
		}
		return ret;
	}
	
	
}
